# This file was automatically generated by SWIG (https://www.swig.org).
# Version 4.4.0
#
# Do not make changes to this file unless you know what you are doing - modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
# Import the low-level C/C++ module
if getattr(globals().get("__spec__"), "parent", None) or __package__ or "." in __name__:
    from . import _mingc
else:
    import _mingc

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "this":
            set(self, name, value)
        elif name == "thisown":
            self.this.own(value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)



def new_intp():
    return _mingc.new_intp()

def copy_intp(value):
    return _mingc.copy_intp(value)

def delete_intp(obj):
    return _mingc.delete_intp(obj)

def intp_assign(obj, value):
    return _mingc.intp_assign(obj, value)

def intp_value(obj):
    return _mingc.intp_value(obj)

def new_floatp():
    return _mingc.new_floatp()

def copy_floatp(value):
    return _mingc.copy_floatp(value)

def delete_floatp(obj):
    return _mingc.delete_floatp(obj)

def floatp_assign(obj, value):
    return _mingc.floatp_assign(obj, value)

def floatp_value(obj):
    return _mingc.floatp_value(obj)
class floatArray(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, nelements):
        _mingc.floatArray_swiginit(self, _mingc.new_floatArray(nelements))
    __swig_destroy__ = _mingc.delete_floatArray

    def __getitem__(self, index):
        return _mingc.floatArray___getitem__(self, index)

    def __setitem__(self, index, value):
        return _mingc.floatArray___setitem__(self, index, value)

    def cast(self):
        return _mingc.floatArray_cast(self)

    @staticmethod
    def frompointer(t):
        return _mingc.floatArray_frompointer(t)

# Register floatArray in _mingc:
_mingc.floatArray_swigregister(floatArray)

def Ming_init():
    return _mingc.Ming_init()

def Ming_cleanup():
    return _mingc.Ming_cleanup()

def Ming_collectGarbage():
    return _mingc.Ming_collectGarbage()

def Ming_useConstants(flag):
    return _mingc.Ming_useConstants(flag)

def Ming_setCubicThreshold(num):
    return _mingc.Ming_setCubicThreshold(num)

def Ming_setScale(scale):
    return _mingc.Ming_setScale(scale)

def Ming_getScale():
    return _mingc.Ming_getScale()

def Ming_useSWFVersion(version):
    return _mingc.Ming_useSWFVersion(version)

def Ming_setSWFCompression(level):
    return _mingc.Ming_setSWFCompression(level)

def SWFMatrix_getScaleX(m):
    return _mingc.SWFMatrix_getScaleX(m)

def SWFMatrix_getRotate0(m):
    return _mingc.SWFMatrix_getRotate0(m)

def SWFMatrix_getRotate1(m):
    return _mingc.SWFMatrix_getRotate1(m)

def SWFMatrix_getScaleY(m):
    return _mingc.SWFMatrix_getScaleY(m)

def SWFMatrix_getTranslateX(m):
    return _mingc.SWFMatrix_getTranslateX(m)

def SWFMatrix_getTranslateY(m):
    return _mingc.SWFMatrix_getTranslateY(m)

def newSWFInput_file(f):
    return _mingc.newSWFInput_file(f)

def newSWFInput_stream(f):
    return _mingc.newSWFInput_stream(f)

def newSWFInput_buffer(buffer, length):
    return _mingc.newSWFInput_buffer(buffer, length)

def newSWFInput_allocedBuffer(buffer, length):
    return _mingc.newSWFInput_allocedBuffer(buffer, length)

def destroySWFInput(input):
    return _mingc.destroySWFInput(input)

def SWFInput_length(input):
    return _mingc.SWFInput_length(input)

def SWFInput_rewind(input):
    return _mingc.SWFInput_rewind(input)

def SWFInput_tell(input):
    return _mingc.SWFInput_tell(input)

def SWFInput_seek(input, offset, whence):
    return _mingc.SWFInput_seek(input, offset, whence)

def SWFInput_eof(input):
    return _mingc.SWFInput_eof(input)

def SWFCharacter_getWidth(character):
    return _mingc.SWFCharacter_getWidth(character)

def SWFCharacter_getHeight(character):
    return _mingc.SWFCharacter_getHeight(character)

def newSWFBitmap_fromInput(input):
    return _mingc.newSWFBitmap_fromInput(input)

def destroySWFBitmap(bitmap):
    return _mingc.destroySWFBitmap(bitmap)

def SWFBitmap_getWidth(b):
    return _mingc.SWFBitmap_getWidth(b)

def SWFBitmap_getHeight(b):
    return _mingc.SWFBitmap_getHeight(b)

def newSWFDBLBitmap(f):
    return _mingc.newSWFDBLBitmap(f)

def newSWFDBLBitmap_fromInput(input):
    return _mingc.newSWFDBLBitmap_fromInput(input)

def newSWFJpegBitmap(f):
    return _mingc.newSWFJpegBitmap(f)

def newSWFJpegBitmap_fromInput(input):
    return _mingc.newSWFJpegBitmap_fromInput(input)

def newSWFJpegWithAlpha(f, alpha):
    return _mingc.newSWFJpegWithAlpha(f, alpha)

def newSWFJpegWithAlpha_fromInput(input, alpha):
    return _mingc.newSWFJpegWithAlpha_fromInput(input, alpha)
SWF_GRADIENT_PAD = _mingc.SWF_GRADIENT_PAD
SWF_GRADIENT_REFLECT = _mingc.SWF_GRADIENT_REFLECT
SWF_GRADIENT_REPEAT = _mingc.SWF_GRADIENT_REPEAT
SWF_GRADIENT_NORMAL = _mingc.SWF_GRADIENT_NORMAL
SWF_GRADIENT_LINEAR = _mingc.SWF_GRADIENT_LINEAR

def newSWFGradient():
    return _mingc.newSWFGradient()

def destroySWFGradient(gradient):
    return _mingc.destroySWFGradient(gradient)

def SWFGradient_addEntry(gradient, ratio, r, g, b, a):
    return _mingc.SWFGradient_addEntry(gradient, ratio, r, g, b, a)

def SWFGradient_setSpreadMode(gradient, mode):
    return _mingc.SWFGradient_setSpreadMode(gradient, mode)

def SWFGradient_setInterpolationMode(gradient, mode):
    return _mingc.SWFGradient_setInterpolationMode(gradient, mode)

def SWFGradient_setFocalPoint(gradient, focalPoint):
    return _mingc.SWFGradient_setFocalPoint(gradient, focalPoint)
SWFFILL_SOLID = _mingc.SWFFILL_SOLID
SWFFILL_GRADIENT = _mingc.SWFFILL_GRADIENT
SWFFILL_LINEAR_GRADIENT = _mingc.SWFFILL_LINEAR_GRADIENT
SWFFILL_RADIAL_GRADIENT = _mingc.SWFFILL_RADIAL_GRADIENT
SWFFILL_BITMAP = _mingc.SWFFILL_BITMAP
SWFFILL_TILED_BITMAP = _mingc.SWFFILL_TILED_BITMAP
SWFFILL_CLIPPED_BITMAP = _mingc.SWFFILL_CLIPPED_BITMAP

def newSWFSolidFillStyle(r, g, b, a):
    return _mingc.newSWFSolidFillStyle(r, g, b, a)

def newSWFGradientFillStyle(gradient, radial):
    return _mingc.newSWFGradientFillStyle(gradient, radial)

def newSWFBitmapFillStyle(bitmap, flags):
    return _mingc.newSWFBitmapFillStyle(bitmap, flags)

def SWFFillStyle_getMatrix(fill):
    return _mingc.SWFFillStyle_getMatrix(fill)

def newSWFLineStyle(width, r, g, b, a):
    return _mingc.newSWFLineStyle(width, r, g, b, a)
SWF_LINESTYLE_CAP_ROUND = _mingc.SWF_LINESTYLE_CAP_ROUND
SWF_LINESTYLE_CAP_NONE = _mingc.SWF_LINESTYLE_CAP_NONE
SWF_LINESTYLE_CAP_SQUARE = _mingc.SWF_LINESTYLE_CAP_SQUARE
SWF_LINESTYLE_JOIN_ROUND = _mingc.SWF_LINESTYLE_JOIN_ROUND
SWF_LINESTYLE_JOIN_BEVEL = _mingc.SWF_LINESTYLE_JOIN_BEVEL
SWF_LINESTYLE_JOIN_MITER = _mingc.SWF_LINESTYLE_JOIN_MITER
SWF_LINESTYLE_FLAG_NOHSCALE = _mingc.SWF_LINESTYLE_FLAG_NOHSCALE
SWF_LINESTYLE_FLAG_NOVSCALE = _mingc.SWF_LINESTYLE_FLAG_NOVSCALE
SWF_LINESTYLE_FLAG_HINTING = _mingc.SWF_LINESTYLE_FLAG_HINTING
SWF_LINESTYLE_FLAG_NOCLOSE = _mingc.SWF_LINESTYLE_FLAG_NOCLOSE
SWF_LINESTYLE_FLAG_ENDCAP_ROUND = _mingc.SWF_LINESTYLE_FLAG_ENDCAP_ROUND
SWF_LINESTYLE_FLAG_ENDCAP_NONE = _mingc.SWF_LINESTYLE_FLAG_ENDCAP_NONE
SWF_LINESTYLE_FLAG_ENDCAP_SQUARE = _mingc.SWF_LINESTYLE_FLAG_ENDCAP_SQUARE

def newSWFLineStyle2(width, r, g, b, a, flags, miterLimit):
    return _mingc.newSWFLineStyle2(width, r, g, b, a, flags, miterLimit)

def newSWFLineStyle2_filled(width, fill, flags, miterLimit):
    return _mingc.newSWFLineStyle2_filled(width, fill, flags, miterLimit)
SWF_SHAPE3 = _mingc.SWF_SHAPE3
SWF_SHAPE4 = _mingc.SWF_SHAPE4
SWF_SHAPE_USESCALINGSTROKES = _mingc.SWF_SHAPE_USESCALINGSTROKES
SWF_SHAPE_USENONSCALINGSTROKES = _mingc.SWF_SHAPE_USENONSCALINGSTROKES

def newSWFShape():
    return _mingc.newSWFShape()

def newSWFShapeFromBitmap(bitmap, flag):
    return _mingc.newSWFShapeFromBitmap(bitmap, flag)

def destroySWFShape(shape):
    return _mingc.destroySWFShape(shape)

def SWFShape_end(shape):
    return _mingc.SWFShape_end(shape)

def SWFShape_useVersion(shape, version):
    return _mingc.SWFShape_useVersion(shape, version)

def SWFShape_getVersion(shape):
    return _mingc.SWFShape_getVersion(shape)

def SWFShape_setRenderHintingFlags(shape, flags):
    return _mingc.SWFShape_setRenderHintingFlags(shape, flags)

def SWFShape_movePenTo(shape, x, y):
    return _mingc.SWFShape_movePenTo(shape, x, y)

def SWFShape_movePen(shape, x, y):
    return _mingc.SWFShape_movePen(shape, x, y)

def SWFShape_getPenX(shape):
    return _mingc.SWFShape_getPenX(shape)

def SWFShape_getPenY(shape):
    return _mingc.SWFShape_getPenY(shape)

def SWFShape_getPen(shape, penX, penY):
    return _mingc.SWFShape_getPen(shape, penX, penY)

def SWFShape_drawLineTo(shape, x, y):
    return _mingc.SWFShape_drawLineTo(shape, x, y)

def SWFShape_drawLine(shape, dx, dy):
    return _mingc.SWFShape_drawLine(shape, dx, dy)

def SWFShape_drawCurveTo(shape, controlx, controly, anchorx, anchory):
    return _mingc.SWFShape_drawCurveTo(shape, controlx, controly, anchorx, anchory)

def SWFShape_drawCurve(shape, controldx, controldy, anchordx, anchordy):
    return _mingc.SWFShape_drawCurve(shape, controldx, controldy, anchordx, anchordy)

def SWFShape_hideLine(shape):
    return _mingc.SWFShape_hideLine(shape)

def SWFShape_addSolidFillStyle(shape, r, g, b, a):
    return _mingc.SWFShape_addSolidFillStyle(shape, r, g, b, a)

def SWFShape_addGradientFillStyle(shape, gradient, flags):
    return _mingc.SWFShape_addGradientFillStyle(shape, gradient, flags)

def SWFShape_addBitmapFillStyle(shape, bitmap, flags):
    return _mingc.SWFShape_addBitmapFillStyle(shape, bitmap, flags)

def SWFShape_setLeftFillStyle(shape, fill):
    return _mingc.SWFShape_setLeftFillStyle(shape, fill)

def SWFShape_setRightFillStyle(shape, fill):
    return _mingc.SWFShape_setRightFillStyle(shape, fill)

def newSWFMorphShape():
    return _mingc.newSWFMorphShape()

def destroySWFMorph(morph):
    return _mingc.destroySWFMorph(morph)

def SWFMorph_getShape1(morph):
    return _mingc.SWFMorph_getShape1(morph)

def SWFMorph_getShape2(morph):
    return _mingc.SWFMorph_getShape2(morph)

def newSWFFont_fromFile(filename):
    return _mingc.newSWFFont_fromFile(filename)

def destroySWFFont(font):
    return _mingc.destroySWFFont(font)

def SWFFont_getStringWidth(font, string):
    return _mingc.SWFFont_getStringWidth(font, string)

def SWFFont_getUTF8StringWidth(font, string):
    return _mingc.SWFFont_getUTF8StringWidth(font, string)

def SWFFont_getWideStringWidth(font, string, len):
    return _mingc.SWFFont_getWideStringWidth(font, string, len)

def SWFFont_getAscent(font):
    return _mingc.SWFFont_getAscent(font)

def SWFFont_getDescent(font):
    return _mingc.SWFFont_getDescent(font)

def SWFFont_getLeading(font):
    return _mingc.SWFFont_getLeading(font)

def SWFFont_getName(font):
    return _mingc.SWFFont_getName(font)

def SWFFont_getGlyphCount(font):
    return _mingc.SWFFont_getGlyphCount(font)

def newSWFFontCollection_fromFile(filename):
    return _mingc.newSWFFontCollection_fromFile(filename)

def SWFFontCollection_getFontCount(collection):
    return _mingc.SWFFontCollection_getFontCount(collection)

def SWFFontCollection_getFont(collection, index):
    return _mingc.SWFFontCollection_getFont(collection, index)

def destroySWFFontCollection(collection):
    return _mingc.destroySWFFontCollection(collection)

def newSWFText():
    return _mingc.newSWFText()

def newSWFText2():
    return _mingc.newSWFText2()

def destroySWFText(text):
    return _mingc.destroySWFText(text)

def SWFText_setFont(text, font):
    return _mingc.SWFText_setFont(text, font)

def SWFText_setHeight(text, height):
    return _mingc.SWFText_setHeight(text, height)

def SWFText_setColor(text, r, g, b, a):
    return _mingc.SWFText_setColor(text, r, g, b, a)

def SWFText_moveTo(text, x, y):
    return _mingc.SWFText_moveTo(text, x, y)

def SWFText_addString(text, string, advance):
    return _mingc.SWFText_addString(text, string, advance)

def SWFText_addUTF8String(text, string, advance):
    return _mingc.SWFText_addUTF8String(text, string, advance)

def SWFText_addWideString(text, string, strlen, advance):
    return _mingc.SWFText_addWideString(text, string, strlen, advance)

def SWFText_setSpacing(text, spacing):
    return _mingc.SWFText_setSpacing(text, spacing)

def SWFText_getStringWidth(text, string):
    return _mingc.SWFText_getStringWidth(text, string)

def SWFText_getUTF8StringWidth(text, string):
    return _mingc.SWFText_getUTF8StringWidth(text, string)

def SWFText_getWideStringWidth(text, string):
    return _mingc.SWFText_getWideStringWidth(text, string)

def SWFText_getAscent(text):
    return _mingc.SWFText_getAscent(text)

def SWFText_getDescent(text):
    return _mingc.SWFText_getDescent(text)

def SWFText_getLeading(text):
    return _mingc.SWFText_getLeading(text)

def newSWFBrowserFont(name):
    return _mingc.newSWFBrowserFont(name)

def destroySWFBrowserFont(browserFont):
    return _mingc.destroySWFBrowserFont(browserFont)

def SWFFontCharacter_addChars(font, string):
    return _mingc.SWFFontCharacter_addChars(font, string)

def SWFFontCharacter_addUTF8Chars(font, string):
    return _mingc.SWFFontCharacter_addUTF8Chars(font, string)
SWFTEXTFIELD_ONMASK = _mingc.SWFTEXTFIELD_ONMASK
SWFTEXTFIELD_OFFMASK = _mingc.SWFTEXTFIELD_OFFMASK
SWFTEXTFIELD_HASFONT = _mingc.SWFTEXTFIELD_HASFONT
SWFTEXTFIELD_HASLENGTH = _mingc.SWFTEXTFIELD_HASLENGTH
SWFTEXTFIELD_HASCOLOR = _mingc.SWFTEXTFIELD_HASCOLOR
SWFTEXTFIELD_NOEDIT = _mingc.SWFTEXTFIELD_NOEDIT
SWFTEXTFIELD_PASSWORD = _mingc.SWFTEXTFIELD_PASSWORD
SWFTEXTFIELD_MULTILINE = _mingc.SWFTEXTFIELD_MULTILINE
SWFTEXTFIELD_WORDWRAP = _mingc.SWFTEXTFIELD_WORDWRAP
SWFTEXTFIELD_HASTEXT = _mingc.SWFTEXTFIELD_HASTEXT
SWFTEXTFIELD_USEFONT = _mingc.SWFTEXTFIELD_USEFONT
SWFTEXTFIELD_HTML = _mingc.SWFTEXTFIELD_HTML
SWFTEXTFIELD_DRAWBOX = _mingc.SWFTEXTFIELD_DRAWBOX
SWFTEXTFIELD_NOSELECT = _mingc.SWFTEXTFIELD_NOSELECT
SWFTEXTFIELD_HASLAYOUT = _mingc.SWFTEXTFIELD_HASLAYOUT
SWFTEXTFIELD_AUTOSIZE = _mingc.SWFTEXTFIELD_AUTOSIZE
SWFTEXTFIELD_ALIGN_LEFT = _mingc.SWFTEXTFIELD_ALIGN_LEFT
SWFTEXTFIELD_ALIGN_RIGHT = _mingc.SWFTEXTFIELD_ALIGN_RIGHT
SWFTEXTFIELD_ALIGN_CENTER = _mingc.SWFTEXTFIELD_ALIGN_CENTER
SWFTEXTFIELD_ALIGN_JUSTIFY = _mingc.SWFTEXTFIELD_ALIGN_JUSTIFY

def newSWFTextField():
    return _mingc.newSWFTextField()

def destroySWFTextField(textField):
    return _mingc.destroySWFTextField(textField)

def SWFTextField_setFont(field, font):
    return _mingc.SWFTextField_setFont(field, font)

def SWFTextField_setBounds(field, width, height):
    return _mingc.SWFTextField_setBounds(field, width, height)

def SWFTextField_setFlags(field, flags):
    return _mingc.SWFTextField_setFlags(field, flags)

def SWFTextField_setColor(field, r, g, b, a):
    return _mingc.SWFTextField_setColor(field, r, g, b, a)

def SWFTextField_setVariableName(field, name):
    return _mingc.SWFTextField_setVariableName(field, name)

def SWFTextField_addString(field, string):
    return _mingc.SWFTextField_addString(field, string)

def SWFTextField_addUTF8String(field, string):
    return _mingc.SWFTextField_addUTF8String(field, string)

def SWFTextField_setHeight(field, height):
    return _mingc.SWFTextField_setHeight(field, height)

def SWFTextField_setFieldHeight(field, height):
    return _mingc.SWFTextField_setFieldHeight(field, height)

def SWFTextField_setLeftMargin(field, leftMargin):
    return _mingc.SWFTextField_setLeftMargin(field, leftMargin)

def SWFTextField_setRightMargin(field, rightMargin):
    return _mingc.SWFTextField_setRightMargin(field, rightMargin)

def SWFTextField_setIndentation(field, indentation):
    return _mingc.SWFTextField_setIndentation(field, indentation)

def SWFTextField_setLineSpacing(field, lineSpacing):
    return _mingc.SWFTextField_setLineSpacing(field, lineSpacing)

def SWFTextField_setPadding(field, padding):
    return _mingc.SWFTextField_setPadding(field, padding)

def SWFTextField_addChars(field, string):
    return _mingc.SWFTextField_addChars(field, string)

def SWFTextField_setAlignment(field, alignment):
    return _mingc.SWFTextField_setAlignment(field, alignment)

def SWFTextField_setLength(field, length):
    return _mingc.SWFTextField_setLength(field, length)

def newSWFSoundStream(file):
    return _mingc.newSWFSoundStream(file)

def newSWFSoundStreamFromFileno(fd):
    return _mingc.newSWFSoundStreamFromFileno(fd)

def newSWFSoundStream_fromInput(input):
    return _mingc.newSWFSoundStream_fromInput(input)

def destroySWFSoundStream(soundStream):
    return _mingc.destroySWFSoundStream(soundStream)

def SWFSoundStream_getDuration(arg1):
    return _mingc.SWFSoundStream_getDuration(arg1)

def SWFSoundStream_setInitialMp3Delay(soundStream, delay):
    return _mingc.SWFSoundStream_setInitialMp3Delay(soundStream, delay)

def newSWFSound(file, flags):
    return _mingc.newSWFSound(file, flags)

def newSWFSoundFromFileno(fd, flags):
    return _mingc.newSWFSoundFromFileno(fd, flags)

def newSWFSound_fromInput(input, flags):
    return _mingc.newSWFSound_fromInput(input, flags)

def newSWFSound_fromSoundStream(stream):
    return _mingc.newSWFSound_fromSoundStream(stream)

def destroySWFSound(sound):
    return _mingc.destroySWFSound(sound)

def SWFSound_setInitialMp3Delay(sound, delay):
    return _mingc.SWFSound_setInitialMp3Delay(sound, delay)
SWF_SOUND_COMPRESSION = _mingc.SWF_SOUND_COMPRESSION
SWF_SOUND_NOT_COMPRESSED = _mingc.SWF_SOUND_NOT_COMPRESSED
SWF_SOUND_ADPCM_COMPRESSED = _mingc.SWF_SOUND_ADPCM_COMPRESSED
SWF_SOUND_MP3_COMPRESSED = _mingc.SWF_SOUND_MP3_COMPRESSED
SWF_SOUND_NOT_COMPRESSED_LE = _mingc.SWF_SOUND_NOT_COMPRESSED_LE
SWF_SOUND_NELLY_COMPRESSED = _mingc.SWF_SOUND_NELLY_COMPRESSED
SWF_SOUND_RATE = _mingc.SWF_SOUND_RATE
SWF_SOUND_5KHZ = _mingc.SWF_SOUND_5KHZ
SWF_SOUND_11KHZ = _mingc.SWF_SOUND_11KHZ
SWF_SOUND_22KHZ = _mingc.SWF_SOUND_22KHZ
SWF_SOUND_44KHZ = _mingc.SWF_SOUND_44KHZ
SWF_SOUND_BITS = _mingc.SWF_SOUND_BITS
SWF_SOUND_8BITS = _mingc.SWF_SOUND_8BITS
SWF_SOUND_16BITS = _mingc.SWF_SOUND_16BITS
SWF_SOUND_CHANNELS = _mingc.SWF_SOUND_CHANNELS
SWF_SOUND_MONO = _mingc.SWF_SOUND_MONO
SWF_SOUND_STEREO = _mingc.SWF_SOUND_STEREO

def SWFSoundInstance_setLoopInPoint(instance, point):
    return _mingc.SWFSoundInstance_setLoopInPoint(instance, point)

def SWFSoundInstance_setLoopOutPoint(instance, point):
    return _mingc.SWFSoundInstance_setLoopOutPoint(instance, point)

def SWFSoundInstance_setLoopCount(instance, count):
    return _mingc.SWFSoundInstance_setLoopCount(instance, count)

def SWFSoundInstance_setNoMultiple(instance):
    return _mingc.SWFSoundInstance_setNoMultiple(instance)

def newSWFCXform(rAdd, gAdd, bAdd, aAdd, rMult, gMult, bMult, aMult):
    return _mingc.newSWFCXform(rAdd, gAdd, bAdd, aAdd, rMult, gMult, bMult, aMult)

def newSWFAddCXform(rAdd, gAdd, bAdd, aAdd):
    return _mingc.newSWFAddCXform(rAdd, gAdd, bAdd, aAdd)

def newSWFMultCXform(rMult, gMult, bMult, aMult):
    return _mingc.newSWFMultCXform(rMult, gMult, bMult, aMult)

def destroySWFCXform(cXform):
    return _mingc.destroySWFCXform(cXform)

def SWFCXform_setColorAdd(cXform, rAdd, gAdd, bAdd, aAdd):
    return _mingc.SWFCXform_setColorAdd(cXform, rAdd, gAdd, bAdd, aAdd)

def SWFCXform_setColorMult(cXform, rMult, gMult, bMult, aMult):
    return _mingc.SWFCXform_setColorMult(cXform, rMult, gMult, bMult, aMult)

def newSWFAction(script):
    return _mingc.newSWFAction(script)

def newSWFAction_fromFile(filename):
    return _mingc.newSWFAction_fromFile(filename)

def SWFAction_compile(action, swfVersion, length):
    return _mingc.SWFAction_compile(action, swfVersion, length)

def destroySWFAction(action):
    return _mingc.destroySWFAction(action)

def SWFAction_getByteCode(action, length):
    return _mingc.SWFAction_getByteCode(action, length)

def newSWFInitAction(action):
    return _mingc.newSWFInitAction(action)

def newSWFInitAction_withId(action, id):
    return _mingc.newSWFInitAction_withId(action, id)
SWFBUTTON_HIT = _mingc.SWFBUTTON_HIT
SWFBUTTON_DOWN = _mingc.SWFBUTTON_DOWN
SWFBUTTON_OVER = _mingc.SWFBUTTON_OVER
SWFBUTTON_UP = _mingc.SWFBUTTON_UP
SWFBUTTONRECORD_HITSTATE = _mingc.SWFBUTTONRECORD_HITSTATE
SWFBUTTONRECORD_DOWNSTATE = _mingc.SWFBUTTONRECORD_DOWNSTATE
SWFBUTTONRECORD_OVERSTATE = _mingc.SWFBUTTONRECORD_OVERSTATE
SWFBUTTONRECORD_UPSTATE = _mingc.SWFBUTTONRECORD_UPSTATE
SWFBUTTON_OVERDOWNTOIDLE = _mingc.SWFBUTTON_OVERDOWNTOIDLE
SWFBUTTON_IDLETOOVERDOWN = _mingc.SWFBUTTON_IDLETOOVERDOWN
SWFBUTTON_OUTDOWNTOIDLE = _mingc.SWFBUTTON_OUTDOWNTOIDLE
SWFBUTTON_OUTDOWNTOOVERDOWN = _mingc.SWFBUTTON_OUTDOWNTOOVERDOWN
SWFBUTTON_OVERDOWNTOOUTDOWN = _mingc.SWFBUTTON_OVERDOWNTOOUTDOWN
SWFBUTTON_OVERDOWNTOOVERUP = _mingc.SWFBUTTON_OVERDOWNTOOVERUP
SWFBUTTON_OVERUPTOOVERDOWN = _mingc.SWFBUTTON_OVERUPTOOVERDOWN
SWFBUTTON_OVERUPTOIDLE = _mingc.SWFBUTTON_OVERUPTOIDLE
SWFBUTTON_IDLETOOVERUP = _mingc.SWFBUTTON_IDLETOOVERUP
SWFBUTTON_MOUSEUPOUTSIDE = _mingc.SWFBUTTON_MOUSEUPOUTSIDE
SWFBUTTON_DRAGOVER = _mingc.SWFBUTTON_DRAGOVER
SWFBUTTON_DRAGOUT = _mingc.SWFBUTTON_DRAGOUT
SWFBUTTON_MOUSEUP = _mingc.SWFBUTTON_MOUSEUP
SWFBUTTON_MOUSEDOWN = _mingc.SWFBUTTON_MOUSEDOWN
SWFBUTTON_MOUSEOUT = _mingc.SWFBUTTON_MOUSEOUT
SWFBUTTON_MOUSEOVER = _mingc.SWFBUTTON_MOUSEOVER

def newSWFButton():
    return _mingc.newSWFButton()

def destroySWFButton(button):
    return _mingc.destroySWFButton(button)

def SWFButton_addCharacter(arg1, arg2, flags):
    return _mingc.SWFButton_addCharacter(arg1, arg2, flags)

def SWFButton_addAction(button, action, flags):
    return _mingc.SWFButton_addAction(button, action, flags)

def SWFButton_addSound(button, action, flags):
    return _mingc.SWFButton_addSound(button, action, flags)

def SWFButton_setMenu(button, flag):
    return _mingc.SWFButton_setMenu(button, flag)

def SWFButton_setScalingGrid(b, x, y, w, h):
    return _mingc.SWFButton_setScalingGrid(b, x, y, w, h)

def SWFButton_removeScalingGrid(b):
    return _mingc.SWFButton_removeScalingGrid(b)

def SWFButtonRecord_setDepth(b, depth):
    return _mingc.SWFButtonRecord_setDepth(b, depth)

def SWFButtonRecord_addFilter(b, f):
    return _mingc.SWFButtonRecord_addFilter(b, f)

def SWFButtonRecord_setBlendMode(b, mode):
    return _mingc.SWFButtonRecord_setBlendMode(b, mode)

def SWFButtonRecord_move(record, x, y):
    return _mingc.SWFButtonRecord_move(record, x, y)

def SWFButtonRecord_moveTo(record, x, y):
    return _mingc.SWFButtonRecord_moveTo(record, x, y)

def SWFButtonRecord_rotate(record, deg):
    return _mingc.SWFButtonRecord_rotate(record, deg)

def SWFButtonRecord_rotateTo(record, deg):
    return _mingc.SWFButtonRecord_rotateTo(record, deg)

def SWFButtonRecord_scale(record, scaleX, scaleY):
    return _mingc.SWFButtonRecord_scale(record, scaleX, scaleY)

def SWFButtonRecord_scaleTo(record, scaleX, scaleY):
    return _mingc.SWFButtonRecord_scaleTo(record, scaleX, scaleY)

def SWFButtonRecord_skewX(record, skewX):
    return _mingc.SWFButtonRecord_skewX(record, skewX)

def SWFButtonRecord_skewXTo(record, skewX):
    return _mingc.SWFButtonRecord_skewXTo(record, skewX)

def SWFButtonRecord_skewY(record, skewY):
    return _mingc.SWFButtonRecord_skewY(record, skewY)

def SWFButtonRecord_skewYTo(record, skewY):
    return _mingc.SWFButtonRecord_skewYTo(record, skewY)
SWFVIDEOSTREAM_MODE_AUTO = _mingc.SWFVIDEOSTREAM_MODE_AUTO
SWFVIDEOSTREAM_MODE_MANUAL = _mingc.SWFVIDEOSTREAM_MODE_MANUAL

def newSWFVideoStream_fromFile(f):
    return _mingc.newSWFVideoStream_fromFile(f)

def newSWFVideoStream_fromInput(input):
    return _mingc.newSWFVideoStream_fromInput(input)

def newSWFVideoStream():
    return _mingc.newSWFVideoStream()

def SWFVideoStream_setDimension(stream, width, height):
    return _mingc.SWFVideoStream_setDimension(stream, width, height)

def SWFVideoStream_getNumFrames(stream):
    return _mingc.SWFVideoStream_getNumFrames(stream)

def SWFVideoStream_hasAudio(stream):
    return _mingc.SWFVideoStream_hasAudio(stream)

def SWFVideoStream_setFrameMode(stream, mode):
    return _mingc.SWFVideoStream_setFrameMode(stream, mode)

def SWFVideoStream_nextFrame(stream):
    return _mingc.SWFVideoStream_nextFrame(stream)

def SWFVideoStream_seek(stream, frame, whence):
    return _mingc.SWFVideoStream_seek(stream, frame, whence)

def destroySWFVideoStream(stream):
    return _mingc.destroySWFVideoStream(stream)

def newSWFSprite():
    return _mingc.newSWFSprite()

def destroySWFSprite(sprite):
    return _mingc.destroySWFSprite(sprite)

def SWFSprite_addBlock(sprite, block):
    return _mingc.SWFSprite_addBlock(sprite, block)

def newSWFPosition(matrix):
    return _mingc.newSWFPosition(matrix)

def destroySWFPosition(position):
    return _mingc.destroySWFPosition(position)

def SWFPosition_skewX(position, x):
    return _mingc.SWFPosition_skewX(position, x)

def SWFPosition_skewXTo(position, x):
    return _mingc.SWFPosition_skewXTo(position, x)

def SWFPosition_skewY(position, y):
    return _mingc.SWFPosition_skewY(position, y)

def SWFPosition_skewYTo(position, y):
    return _mingc.SWFPosition_skewYTo(position, y)

def SWFPosition_scaleX(position, x):
    return _mingc.SWFPosition_scaleX(position, x)

def SWFPosition_scaleXTo(position, x):
    return _mingc.SWFPosition_scaleXTo(position, x)

def SWFPosition_scaleY(position, y):
    return _mingc.SWFPosition_scaleY(position, y)

def SWFPosition_scaleYTo(position, y):
    return _mingc.SWFPosition_scaleYTo(position, y)

def SWFPosition_scaleXY(position, x, y):
    return _mingc.SWFPosition_scaleXY(position, x, y)

def SWFPosition_scaleXYTo(position, x, y):
    return _mingc.SWFPosition_scaleXYTo(position, x, y)

def SWFPosition_getMatrix(p):
    return _mingc.SWFPosition_getMatrix(p)

def SWFPosition_setMatrix(p, a, b, c, d, x, y):
    return _mingc.SWFPosition_setMatrix(p, a, b, c, d, x, y)

def SWFPosition_rotate(position, degrees):
    return _mingc.SWFPosition_rotate(position, degrees)

def SWFPosition_rotateTo(position, degrees):
    return _mingc.SWFPosition_rotateTo(position, degrees)

def SWFPosition_move(position, x, y):
    return _mingc.SWFPosition_move(position, x, y)

def SWFPosition_moveTo(position, x, y):
    return _mingc.SWFPosition_moveTo(position, x, y)

def SWFPosition_getRotation(position):
    return _mingc.SWFPosition_getRotation(position)

def SWFPosition_getX(position):
    return _mingc.SWFPosition_getX(position)

def SWFPosition_getY(position):
    return _mingc.SWFPosition_getY(position)

def SWFPosition_getXY(position, outX, outY):
    return _mingc.SWFPosition_getXY(position, outX, outY)

def SWFPosition_getXScale(position):
    return _mingc.SWFPosition_getXScale(position)

def SWFPosition_getYScale(position):
    return _mingc.SWFPosition_getYScale(position)

def SWFPosition_getXYScale(position, outXScale, outYScale):
    return _mingc.SWFPosition_getXYScale(position, outXScale, outYScale)

def SWFPosition_getXSkew(position):
    return _mingc.SWFPosition_getXSkew(position)

def SWFPosition_getYSkew(position):
    return _mingc.SWFPosition_getYSkew(position)

def SWFPosition_getXYSkew(position, outXSkew, outYSkew):
    return _mingc.SWFPosition_getXYSkew(position, outXSkew, outYSkew)

def SWFDisplayItem_getCharacter(item):
    return _mingc.SWFDisplayItem_getCharacter(item)

def SWFDisplayItem_endMask(item):
    return _mingc.SWFDisplayItem_endMask(item)

def SWFDisplayItem_move(item, x, y):
    return _mingc.SWFDisplayItem_move(item, x, y)

def SWFDisplayItem_moveTo(item, x, y):
    return _mingc.SWFDisplayItem_moveTo(item, x, y)

def SWFDisplayItem_rotate(item, degrees):
    return _mingc.SWFDisplayItem_rotate(item, degrees)

def SWFDisplayItem_rotateTo(item, degrees):
    return _mingc.SWFDisplayItem_rotateTo(item, degrees)

def SWFDisplayItem_scale(item, xScale, yScale):
    return _mingc.SWFDisplayItem_scale(item, xScale, yScale)

def SWFDisplayItem_scaleTo(item, xScale, yScale):
    return _mingc.SWFDisplayItem_scaleTo(item, xScale, yScale)

def SWFDisplayItem_skewX(item, x):
    return _mingc.SWFDisplayItem_skewX(item, x)

def SWFDisplayItem_skewXTo(item, x):
    return _mingc.SWFDisplayItem_skewXTo(item, x)

def SWFDisplayItem_skewY(item, y):
    return _mingc.SWFDisplayItem_skewY(item, y)

def SWFDisplayItem_skewYTo(item, y):
    return _mingc.SWFDisplayItem_skewYTo(item, y)

def SWFDisplayItem_getPosition(item, x, y):
    return _mingc.SWFDisplayItem_getPosition(item, x, y)

def SWFDisplayItem_getRotation(item, degrees):
    return _mingc.SWFDisplayItem_getRotation(item, degrees)

def SWFDisplayItem_getScale(item, xScale, yScale):
    return _mingc.SWFDisplayItem_getScale(item, xScale, yScale)

def SWFDisplayItem_getSkew(item, xSkew, ySkew):
    return _mingc.SWFDisplayItem_getSkew(item, xSkew, ySkew)

def SWFDisplayItem_getMatrix(item):
    return _mingc.SWFDisplayItem_getMatrix(item)

def SWFDisplayItem_setMatrix(i, a, b, c, d, x, y):
    return _mingc.SWFDisplayItem_setMatrix(i, a, b, c, d, x, y)

def SWFDisplayItem_getDepth(item):
    return _mingc.SWFDisplayItem_getDepth(item)

def SWFDisplayItem_setDepth(item, depth):
    return _mingc.SWFDisplayItem_setDepth(item, depth)

def SWFDisplayItem_remove(item):
    return _mingc.SWFDisplayItem_remove(item)

def SWFDisplayItem_setName(item, name):
    return _mingc.SWFDisplayItem_setName(item, name)

def SWFDisplayItem_setMaskLevel(item, masklevel):
    return _mingc.SWFDisplayItem_setMaskLevel(item, masklevel)

def SWFDisplayItem_setRatio(item, ratio):
    return _mingc.SWFDisplayItem_setRatio(item, ratio)

def SWFDisplayItem_setCXform(item, cXform):
    return _mingc.SWFDisplayItem_setCXform(item, cXform)

def SWFDisplayItem_setColorAdd(item, r, g, b, a):
    return _mingc.SWFDisplayItem_setColorAdd(item, r, g, b, a)

def SWFDisplayItem_setColorMult(item, r, g, b, a):
    return _mingc.SWFDisplayItem_setColorMult(item, r, g, b, a)
SWFACTION_ONLOAD = _mingc.SWFACTION_ONLOAD
SWFACTION_ENTERFRAME = _mingc.SWFACTION_ENTERFRAME
SWFACTION_UNLOAD = _mingc.SWFACTION_UNLOAD
SWFACTION_MOUSEMOVE = _mingc.SWFACTION_MOUSEMOVE
SWFACTION_MOUSEDOWN = _mingc.SWFACTION_MOUSEDOWN
SWFACTION_MOUSEUP = _mingc.SWFACTION_MOUSEUP
SWFACTION_KEYDOWN = _mingc.SWFACTION_KEYDOWN
SWFACTION_KEYUP = _mingc.SWFACTION_KEYUP
SWFACTION_DATA = _mingc.SWFACTION_DATA

def SWFDisplayItem_addAction(item, action, flags):
    return _mingc.SWFDisplayItem_addAction(item, action, flags)

def SWFDisplayItem_cacheAsBitmap(item, flag):
    return _mingc.SWFDisplayItem_cacheAsBitmap(item, flag)
SWFBLEND_MODE_NULL = _mingc.SWFBLEND_MODE_NULL
SWFBLEND_MODE_NORMAL = _mingc.SWFBLEND_MODE_NORMAL
SWFBLEND_MODE_LAYER = _mingc.SWFBLEND_MODE_LAYER
SWFBLEND_MODE_MULT = _mingc.SWFBLEND_MODE_MULT
SWFBLEND_MODE_SCREEN = _mingc.SWFBLEND_MODE_SCREEN
SWFBLEND_MODE_LIGHTEN = _mingc.SWFBLEND_MODE_LIGHTEN
SWFBLEND_MODE_DARKEN = _mingc.SWFBLEND_MODE_DARKEN
SWFBLEND_MODE_DIFF = _mingc.SWFBLEND_MODE_DIFF
SWFBLEND_MODE_ADD = _mingc.SWFBLEND_MODE_ADD
SWFBLEND_MODE_SUB = _mingc.SWFBLEND_MODE_SUB
SWFBLEND_MODE_INV = _mingc.SWFBLEND_MODE_INV
SWFBLEND_MODE_ALPHA = _mingc.SWFBLEND_MODE_ALPHA
SWFBLEND_MODE_ERASE = _mingc.SWFBLEND_MODE_ERASE
SWFBLEND_MODE_OVERLAY = _mingc.SWFBLEND_MODE_OVERLAY
SWFBLEND_MODE_HARDLIGHT = _mingc.SWFBLEND_MODE_HARDLIGHT

def SWFDisplayItem_setBlendMode(item, mode):
    return _mingc.SWFDisplayItem_setBlendMode(item, mode)

def newSWFFill(fillstyle):
    return _mingc.newSWFFill(fillstyle)

def destroySWFFill(fill):
    return _mingc.destroySWFFill(fill)

def SWFFill_getFillStyle(fill):
    return _mingc.SWFFill_getFillStyle(fill)

def SWFFill_skewX(fill, x):
    return _mingc.SWFFill_skewX(fill, x)

def SWFFill_skewXTo(fill, x):
    return _mingc.SWFFill_skewXTo(fill, x)

def SWFFill_skewY(fill, y):
    return _mingc.SWFFill_skewY(fill, y)

def SWFFill_skewYTo(fill, y):
    return _mingc.SWFFill_skewYTo(fill, y)

def SWFFill_scaleX(fill, x):
    return _mingc.SWFFill_scaleX(fill, x)

def SWFFill_scaleXTo(fill, x):
    return _mingc.SWFFill_scaleXTo(fill, x)

def SWFFill_scaleY(fill, y):
    return _mingc.SWFFill_scaleY(fill, y)

def SWFFill_scaleYTo(fill, y):
    return _mingc.SWFFill_scaleYTo(fill, y)

def SWFFill_scaleXY(fill, x, y):
    return _mingc.SWFFill_scaleXY(fill, x, y)

def SWFFill_scaleXYTo(fill, x, y):
    return _mingc.SWFFill_scaleXYTo(fill, x, y)

def SWFFill_rotate(fill, degrees):
    return _mingc.SWFFill_rotate(fill, degrees)

def SWFFill_rotateTo(fill, degrees):
    return _mingc.SWFFill_rotateTo(fill, degrees)

def SWFFill_move(fill, x, y):
    return _mingc.SWFFill_move(fill, x, y)

def SWFFill_moveTo(fill, x, y):
    return _mingc.SWFFill_moveTo(fill, x, y)

def SWFFill_setMatrix(fill, a, b, c, d, x, y):
    return _mingc.SWFFill_setMatrix(fill, a, b, c, d, x, y)

def SWFShape_setLine(shape, width, r, g, b, a):
    return _mingc.SWFShape_setLine(shape, width, r, g, b, a)

def SWFShape_setLine2Filled(shape, width, fill, flags, miterLimit):
    return _mingc.SWFShape_setLine2Filled(shape, width, fill, flags, miterLimit)

def SWFShape_setLine2(shape, width, r, g, b, a, flags, miterLimit):
    return _mingc.SWFShape_setLine2(shape, width, r, g, b, a, flags, miterLimit)

def SWFShape_addSolidFill(shape, r, g, b, a):
    return _mingc.SWFShape_addSolidFill(shape, r, g, b, a)

def SWFShape_addGradientFill(shape, gradient, flags):
    return _mingc.SWFShape_addGradientFill(shape, gradient, flags)

def SWFShape_addBitmapFill(shape, bitmap, flags):
    return _mingc.SWFShape_addBitmapFill(shape, bitmap, flags)

def SWFShape_setLeftFill(shape, fill):
    return _mingc.SWFShape_setLeftFill(shape, fill)

def SWFShape_setRightFill(shape, fill):
    return _mingc.SWFShape_setRightFill(shape, fill)

def SWFShape_drawArc(shape, r, startAngle, endAngle):
    return _mingc.SWFShape_drawArc(shape, r, startAngle, endAngle)

def SWFShape_drawCircle(shape, r):
    return _mingc.SWFShape_drawCircle(shape, r)

def SWFShape_drawGlyph(shape, font, c):
    return _mingc.SWFShape_drawGlyph(shape, font, c)

def SWFShape_drawSizedGlyph(shape, font, c, size):
    return _mingc.SWFShape_drawSizedGlyph(shape, font, c, size)

def SWFShape_drawCubic(shape, bx, by, cx, cy, dx, dy):
    return _mingc.SWFShape_drawCubic(shape, bx, by, cx, cy, dx, dy)

def SWFShape_drawCubicTo(shape, bx, by, cx, cy, dx, dy):
    return _mingc.SWFShape_drawCubicTo(shape, bx, by, cx, cy, dx, dy)

def SWFShape_drawCharacterBounds(shape, character):
    return _mingc.SWFShape_drawCharacterBounds(shape, character)

def newSWFMovieClip():
    return _mingc.newSWFMovieClip()

def destroySWFMovieClip(movieClip):
    return _mingc.destroySWFMovieClip(movieClip)

def SWFMovieClip_setNumberOfFrames(clip, frames):
    return _mingc.SWFMovieClip_setNumberOfFrames(clip, frames)

def SWFMovieClip_nextFrame(clip):
    return _mingc.SWFMovieClip_nextFrame(clip)

def SWFMovieClip_labelFrame(clip, label):
    return _mingc.SWFMovieClip_labelFrame(clip, label)

def SWFMovieClip_add(clip, block):
    return _mingc.SWFMovieClip_add(clip, block)

def SWFMovieClip_remove(clip, item):
    return _mingc.SWFMovieClip_remove(clip, item)

def SWFMovieClip_setSoundStreamAt(clip, sound, rate, skipSound):
    return _mingc.SWFMovieClip_setSoundStreamAt(clip, sound, rate, skipSound)

def SWFMovieClip_setSoundStream(clip, sound, rate):
    return _mingc.SWFMovieClip_setSoundStream(clip, sound, rate)

def SWFMovieClip_startSound(clip, sound):
    return _mingc.SWFMovieClip_startSound(clip, sound)

def SWFMovieClip_stopSound(clip, sound):
    return _mingc.SWFMovieClip_stopSound(clip, sound)

def destroySWFPrebuiltClip(clip):
    return _mingc.destroySWFPrebuiltClip(clip)

def newSWFPrebuiltClip_fromFile(filename):
    return _mingc.newSWFPrebuiltClip_fromFile(filename)

def newSWFPrebuiltClip_fromInput(input):
    return _mingc.newSWFPrebuiltClip_fromInput(input)

def newSWFBinaryData(blob, length):
    return _mingc.newSWFBinaryData(blob, length)

def newSWFMovie():
    return _mingc.newSWFMovie()

def newSWFMovieWithVersion(version):
    return _mingc.newSWFMovieWithVersion(version)

def destroySWFMovie(movie):
    return _mingc.destroySWFMovie(movie)

def SWFMovie_setRate(movie, rate):
    return _mingc.SWFMovie_setRate(movie, rate)

def SWFMovie_getRate(movie):
    return _mingc.SWFMovie_getRate(movie)

def SWFMovie_setDimension(movie, x, y):
    return _mingc.SWFMovie_setDimension(movie, x, y)

def SWFMovie_setNumberOfFrames(movie, frames):
    return _mingc.SWFMovie_setNumberOfFrames(movie, frames)

def SWFMovie_addExport(movie, block, name):
    return _mingc.SWFMovie_addExport(movie, block, name)

def SWFMovie_setBackground(movie, r, g, b):
    return _mingc.SWFMovie_setBackground(movie, r, g, b)

def SWFMovie_setSoundStream(movie, sound):
    return _mingc.SWFMovie_setSoundStream(movie, sound)

def SWFMovie_setSoundStreamAt(movie, stream, skip):
    return _mingc.SWFMovie_setSoundStreamAt(movie, stream, skip)

def SWFMovie_startSound(movie, sound):
    return _mingc.SWFMovie_startSound(movie, sound)

def SWFMovie_stopSound(movie, sound):
    return _mingc.SWFMovie_stopSound(movie, sound)

def SWFMovie_add(movie, block):
    return _mingc.SWFMovie_add(movie, block)

def SWFMovie_remove(movie, item):
    return _mingc.SWFMovie_remove(movie, item)

def SWFMovie_nextFrame(movie):
    return _mingc.SWFMovie_nextFrame(movie)

def SWFMovie_labelFrame(movie, label):
    return _mingc.SWFMovie_labelFrame(movie, label)

def SWFMovie_output(movie, method, data):
    return _mingc.SWFMovie_output(movie, method, data)

def SWFMovie_save(movie, filename):
    return _mingc.SWFMovie_save(movie, filename)

def SWFMovie_protect(movie, password):
    return _mingc.SWFMovie_protect(movie, password)

def SWFMovie_setNetworkAccess(movie, flag):
    return _mingc.SWFMovie_setNetworkAccess(movie, flag)

def SWFMovie_addMetadata(movie, xml):
    return _mingc.SWFMovie_addMetadata(movie, xml)

def SWFMovie_setScriptLimits(movie, maxRecursion, timeout):
    return _mingc.SWFMovie_setScriptLimits(movie, maxRecursion, timeout)

def SWFMovie_setTabIndex(movie, depth, i):
    return _mingc.SWFMovie_setTabIndex(movie, depth, i)

def SWFMovie_importFont(movie, filename, name):
    return _mingc.SWFMovie_importFont(movie, filename, name)

def SWFMovie_addFont(movie, font):
    return _mingc.SWFMovie_addFont(movie, font)

def SWFMovie_importCharacter(movie, filename, name):
    return _mingc.SWFMovie_importCharacter(movie, filename, name)

def SWFMovie_assignSymbol(m, character, name):
    return _mingc.SWFMovie_assignSymbol(m, character, name)

def SWFMovie_defineScene(m, offset, name):
    return _mingc.SWFMovie_defineScene(m, offset, name)

def SWFMovie_namedAnchor(movie, label):
    return _mingc.SWFMovie_namedAnchor(movie, label)

def SWFMovie_writeExports(movie):
    return _mingc.SWFMovie_writeExports(movie)
SWFFILTER_MODE_INNER = _mingc.SWFFILTER_MODE_INNER
SWFFILTER_MODE_KO = _mingc.SWFFILTER_MODE_KO
SWFFILTER_MODE_COMPOSITE = _mingc.SWFFILTER_MODE_COMPOSITE
SWFFILTER_MODE_ONTOP = _mingc.SWFFILTER_MODE_ONTOP
SWFFILTER_FLAG_CLAMP = _mingc.SWFFILTER_FLAG_CLAMP
SWFFILTER_FLAG_PRESERVE_ALPHA = _mingc.SWFFILTER_FLAG_PRESERVE_ALPHA
SWFFILTER_TYPE_DROPSHADOW = _mingc.SWFFILTER_TYPE_DROPSHADOW
SWFFILTER_TYPE_BLUR = _mingc.SWFFILTER_TYPE_BLUR
SWFFILTER_TYPE_GLOW = _mingc.SWFFILTER_TYPE_GLOW
SWFFILTER_TYPE_BEVEL = _mingc.SWFFILTER_TYPE_BEVEL
SWFFILTER_TYPE_GRADIENTGLOW = _mingc.SWFFILTER_TYPE_GRADIENTGLOW
SWFFILTER_TYPE_CONVOLUTION = _mingc.SWFFILTER_TYPE_CONVOLUTION
SWFFILTER_TYPE_COLORMATRIX = _mingc.SWFFILTER_TYPE_COLORMATRIX
SWFFILTER_TYPE_GRADIENTBEVEL = _mingc.SWFFILTER_TYPE_GRADIENTBEVEL
class SWFColor(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    red = property(_mingc.SWFColor_red_get, _mingc.SWFColor_red_set)
    green = property(_mingc.SWFColor_green_get, _mingc.SWFColor_green_set)
    blue = property(_mingc.SWFColor_blue_get, _mingc.SWFColor_blue_set)
    alpha = property(_mingc.SWFColor_alpha_get, _mingc.SWFColor_alpha_set)

    def __init__(self):
        _mingc.SWFColor_swiginit(self, _mingc.new_SWFColor())
    __swig_destroy__ = _mingc.delete_SWFColor

# Register SWFColor in _mingc:
_mingc.SWFColor_swigregister(SWFColor)

def newSWFShadow(angle, distance, strength):
    return _mingc.newSWFShadow(angle, distance, strength)

def destroySWFShadow(s):
    return _mingc.destroySWFShadow(s)

def newSWFBlur(blurX, blurY, passes):
    return _mingc.newSWFBlur(blurX, blurY, passes)

def destroySWFBlur(b):
    return _mingc.destroySWFBlur(b)

def newSWFFilterMatrix(cols, rows, vals):
    return _mingc.newSWFFilterMatrix(cols, rows, vals)

def destroySWFFilterMatrix(m):
    return _mingc.destroySWFFilterMatrix(m)

def destroySWFFilter(filter):
    return _mingc.destroySWFFilter(filter)

def newColorMatrixFilter(matrix):
    return _mingc.newColorMatrixFilter(matrix)

def newConvolutionFilter(matrix, divisor, bias, color, flags):
    return _mingc.newConvolutionFilter(matrix, divisor, bias, color, flags)

def newGradientBevelFilter(gradient, blur, shadow, flags):
    return _mingc.newGradientBevelFilter(gradient, blur, shadow, flags)

def newGradientGlowFilter(gradient, blur, shadow, flags):
    return _mingc.newGradientGlowFilter(gradient, blur, shadow, flags)

def newBevelFilter(shadowColor, highlightColor, blur, shadow, flags):
    return _mingc.newBevelFilter(shadowColor, highlightColor, blur, shadow, flags)

def newGlowFilter(color, blur, strength, flags):
    return _mingc.newGlowFilter(color, blur, strength, flags)

def newBlurFilter(blur):
    return _mingc.newBlurFilter(blur)

def newDropShadowFilter(color, blur, shadow, flags):
    return _mingc.newDropShadowFilter(color, blur, shadow, flags)

def SWFDisplayItem_addFilter(item, filter):
    return _mingc.SWFDisplayItem_addFilter(item, filter)

